import numpy as np

from .symm_util import bond_angle, is_bond_linear, rotation_matrix, axis_rotation_matrix, center_of_mass_shift, clean_cn_dict, debug_plot, is_planar
from .write_util import write_symmetry_axes
from ase.data import atomic_numbers, atomic_masses


def is_linear(bond_dict):
    for atom1 in bond_dict:
        for atom2 in bond_dict[atom1]:
            for atom3 in bond_dict[atom2]:
                if not atom1 == atom3:
                    c1 = atom1.coordinate
                    c2 = atom2.coordinate
                    c3 = atom3.coordinate
                    v1 = c2 - c1
                    v2 = c2 - c3
                    theta = bond_angle(v1, v2)
                    if not is_bond_linear(theta):
                        return False
    return True


def inversion_center(symbols, coords, tol=1):
    inversion = False
    inverted_coords = np.array([[-x, -y, -z] for x, y, z in coords])
    for idx1, coord1 in enumerate(coords):
        found = False
        for idx2, coord2 in enumerate(inverted_coords):
            if np.allclose(coord1, coord2, atol=tol) and symbols[idx1] == symbols[idx2]:
                found = True
                break
        inversion = found
    return inversion


def cn_axis(symbols, coords, tol=1):

    atnums = list(map(atomic_numbers.__getitem__, symbols))
    masses = np.array(list(map(atomic_masses.__getitem__, atnums)))
    # check for Cn axis up to n=10
    check_n = np.arange(2, 10)

    Cn_dict = {}

    # Loop over all possible Cn axes
    for n in check_n:
        # compute the angles alpha beta and gamma
        theta = 2 * np.pi / n

        R_x = rotation_matrix(theta, 0, 0)
        R_y = rotation_matrix(0, theta, 0)
        R_z = rotation_matrix(0, 0, theta)

        # rotate the coordinates for each axis

        rot_x = coords @ R_x
        rot_y = coords @ R_y
        rot_z = coords @ R_z
        rot_x = center_of_mass_shift(rot_x, masses, None)
        rot_x = center_of_mass_shift(rot_y, masses, None)
        rot_x = center_of_mass_shift(rot_z, masses, None)

        # check if the rotated coordinates are the same as the original coordinates

        cn_found = True
        for idx1, coord in enumerate(coords):
            found = False
            for idx2, rot_coord in enumerate(rot_x):
                if np.allclose(coord, rot_coord, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break
            cn_found = found
            if cn_found == False:
                break

        if cn_found:
            if n not in Cn_dict:
                Cn_dict[n] = ['z']
            else:
                Cn_dict[n].append('z')

        cn_found = True
        for idx1, coord in enumerate(coords):
            found = False
            for idx2, rot_coord in enumerate(rot_y):
                if np.allclose(coord, rot_coord, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break
            cn_found = found
            if cn_found == False:
                break

        if cn_found:
            if n not in Cn_dict:
                Cn_dict[n] = ['y']
            else:
                Cn_dict[n].append('y')

        cn_found = True
        for idx1, coord in enumerate(coords):
            found = False
            for idx2, rot_coord in enumerate(rot_z):
                if np.allclose(coord, rot_coord, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break
            cn_found = found
            if cn_found == False:
                break

        if cn_found:
            if n not in Cn_dict:
                Cn_dict[n] = ['x']
            else:
                Cn_dict[n].append('x')

    return Cn_dict



def sigma_h(symbols, coords, high_symmetry_axis, tol=1):
    atnums = list(map(atomic_numbers.__getitem__, symbols))
    masses = np.array(list(map(atomic_masses.__getitem__, atnums)))
    mirrored_coords = np.array(list(map(lambda x: x - 2 * (x @ high_symmetry_axis) * high_symmetry_axis, coords)))
    mirrored_coords = center_of_mass_shift(mirrored_coords, masses, None)

    # debug_plot(coords, mirrored_coords, high_symmetry_axis)

    sh_found = True
    for idx1, coord1 in enumerate(coords):
        found = False
        for idx2, coord2 in enumerate(mirrored_coords):
            if np.allclose(coord1, coord2, atol=tol) and symbols[idx1] == symbols[idx2]:
                found = True

        sh_found = found

    return sh_found


def sample_mirror_planes(axis, num_samples=10):
    axis = np.array(axis, dtype=float)
    axis = axis / np.linalg.norm(axis)  # Normalize the axis vector
    
    # Choose two perpendicular vectors in the plane parallel to the axis
    if axis[0] != 0:
        ref_vec = np.array([0, 1, 0])
    else:
        ref_vec = np.array([1, 0, 0])
    
    second_vec = np.cross(axis, ref_vec)
    second_vec /= np.linalg.norm(second_vec)
    
    angles = np.linspace(0, 2 * np.pi, num_samples)

    def normal(angle):
        return np.cos(angle) * ref_vec + np.sin(angle) * second_vec

    normals = list(map(normal, angles))
    
    return normals

def sigma_v(symbols, coords, high_symmetry_axis, tol):
    atnums = list(map(atomic_numbers.__getitem__, symbols))
    masses = np.array(list(map(atomic_masses.__getitem__, atnums)))
    planes = sample_mirror_planes(np.array(high_symmetry_axis), num_samples=100)

    count = 0
    for plane in planes:
        plane /= np.linalg.norm(plane)
        mirrored_coords = np.array(list(map(lambda x: x - 2 * (x @ plane) * plane, coords)))
        mirrored_coords = center_of_mass_shift(coords, masses, None)

        sv_found = True
        for idx1, coord1 in enumerate(coords):
            found = False
            for idx2, coord2 in enumerate(mirrored_coords):
                if np.allclose(coord1, coord2, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break

            sv_found = found
            if not sv_found:
                break

        if sv_found:
            count += 1

    return count > 0


def hemisphere_sample(num_points, hemisphere='upper'):
    theta = np.linspace(0, 2 * np.pi, num_points)
    # phi = np.random.uniform(0, np.pi / 2, num_points) if hemisphere == 'upper' else np.random.uniform(np.pi / 2, np.pi, num_points)
    # phi = np.linspace(0, np.pi / 2, num_points) if hemisphere == 'upper' else np.linspace(np.pi / 2, np.pi, num_points)
    phi = np.linspace(0, 2 * np.pi, num_points)

    x = np.sin(phi) * np.cos(theta)
    y = np.sin(phi) * np.sin(theta)
    z = np.cos(phi)

    return np.column_stack((x, y, z))


def plane_sampling(normal, position, size=10, resolution=10):
    normal = np.array(normal, dtype=float)
    position = np.array(position, dtype=float)

    # Normalize the normal vector
    normal /= np.linalg.norm(normal)

    # Create two orthogonal vectors to form a basis for the plane
    if np.allclose(normal, [1, 0, 0]):  # Edge case: normal aligns with x-axis
        tangent1 = np.array([0, 1, 0])
    else:
        tangent1 = np.cross(normal, [1, 0, 0])
        tangent1 /= np.linalg.norm(tangent1)

    tangent2 = np.cross(normal, tangent1)

    # Generate a grid of points
    linspace = np.linspace(-size / 2, size / 2, resolution)
    xx, yy = np.meshgrid(linspace, linspace)

    # Compute 3D points on the plane
    points = position + xx[..., None] * tangent1 + yy[..., None] * tangent2
    return points.reshape(-1, 3)


def find_perp_rotation_axis(axis, Cn, symbols, coordinates, tol=1):
    axes = plane_sampling(axis, np.array([0.,0.,0.0]))

    atnums = list(map(atomic_numbers.__getitem__, symbols))
    masses = np.array(list(map(atomic_masses.__getitem__, atnums)))

    if tol < 0.5:
        tol = 0.5

    count = 0
    for ax in axes:
        theta = np.pi

        R = axis_rotation_matrix(ax, theta)

        rotated_coordinates = coordinates @ R
        rotated_coordinates = center_of_mass_shift(rotated_coordinates, masses, None)

        rot_found = True
        for idx1, coord1 in enumerate(coordinates):

            found = False
            for idx2, coord2 in enumerate(rotated_coordinates):

                if np.allclose(coord1, coord2, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break

            rot_found = found
            if not rot_found:
                break
        if rot_found:
            count += 1

    return count >= Cn


def find_high_symmetry_axes(symbols, coordinates, tol):
    cns = np.arange(2, 10)
    axes = hemisphere_sample(361)
    # axes = hemisphere_sample(1001)

    Cn_dict = {}

    rotation_order = 1
    hsa = None

    for axis in axes:
        for n in cns:

            theta = 2 * np.pi / n
            R = axis_rotation_matrix(axis, theta)
            # rotate the coordinates for each axis
            rotated_coordinates = coordinates @ R

            rot_found = True

            for idx1, coord1 in enumerate(coordinates):

                found = False
                for idx2, coord2 in enumerate(rotated_coordinates):

                    if np.allclose(coord1, coord2, atol=tol) and symbols[idx1] == symbols[idx2]:
                        found = True
                        break

                rot_found = found
                if not rot_found:
                    break

            if rot_found:

                if n not in Cn_dict:
                    Cn_dict[n] = [axis]
                else:
                    Cn_dict[n].append(axis)

                if n > rotation_order:
                    rotation_order = n
                    hsa = axis

    return rotation_order, hsa, clean_cn_dict(Cn_dict)


def improper_rotations(Cn, hsa, symbols, coordinates, tol=1):
    # improper rotation i.e. C2 on the hsa and then c2 perpendicular
    # find sample the plane perpendicular and then perform two rotations by 2 * pi / 2 radians
    atnums = list(map(atomic_numbers.__getitem__, symbols))
    masses = np.array(list(map(atomic_masses.__getitem__, atnums)))

    perp_planes = plane_sampling(hsa, np.zeros(3))

    for plane in perp_planes:

        rot1 = axis_rotation_matrix(hsa, np.pi)
        rot2 = axis_rotation_matrix(plane, np.pi)

        rotated_coordinates = coordinates @ rot1 @ rot2
        rotated_coordinates = center_of_mass_shift(rotated_coordinates, masses, None)

        S2n_found = True
        for idx1, atom1 in enumerate(coordinates):
            found = False
            for idx2, atom2 in enumerate(rotated_coordinates):
                if np.allclose(atom1, atom2, atol=tol) and symbols[idx1] == symbols[idx2]:
                    found = True
                    break

            S2n_found = found

            if S2n_found:
                return True
            else:
                break

    return False


def analyse(symbols, coords, bond_dict, tol, f, quiet=False):
    if not quiet:
        f.write('{:-^80}\n\n'.format(' Computing (pseudo)-point group symmetry '))
    if is_linear(bond_dict):
        if not quiet:
            f.write('Linear molecule\n')
        if inversion_center(symbols, coords, tol=tol):
            if not quiet:
                f.write('Inversion center\n')
            return 'Dinfh'
        else:
            if not quiet:
                f.write('No Inversion center\n')
            return 'Cinfh'
    if not quiet:
        f.write("Non-linear molecule\n")

    Cn, hsa, Cn_dict = find_high_symmetry_axes(symbols, coords, tol=tol)
    if hsa is not None:
        if not quiet:
            f.write("High Symmetry Axis found:\n\tRotation order: C{}\n\tHigh symmetry axis: {:11.7f} {:11.7f} {:11.7f} Å\n".format(Cn, *hsa))

    count = 0
    for k, v in Cn_dict.items():
        if k == Cn:
            count += len(v)

    if not quiet:
        write_symmetry_axes(Cn_dict, f)

    if count >= 2:
        if not quiet:
            f.write("Two or more Cn where n > 2\n")
        if inversion_center(symbols, coords, tol=tol):
            if not quiet:
                f.write("Inversion center\n")
            if 5 in Cn_dict:
                if not quiet:
                    f.write("C5 axis found\n")
                return "Ih"
            else:
                return "Oh"
        else:
            return "Td"

    if not quiet:
        f.write("Less than 2 Cn where n > 2\n")

    if len(Cn_dict) == 0:
        if not quiet:
            f.write('No Cn axis found\n')
        # check if the molecule is planar
        if is_planar(coords, tol=tol):
            if not quiet:
                f.write('sigma_h found\n')
            return 'Cs'

        if inversion_center(symbols, coords, tol=tol):
            if not quiet:
                f.write('Inversion center found\n')
            return 'Ci'
        if not quiet:
            f.write('No inversion center\n')
        return 'C1'
    
    # find the perpendicular C2s
    if hsa is not None:
        if find_perp_rotation_axis(hsa, Cn, symbols, coords, tol=tol):
                if not quiet:
                    f.write('More than {} C2 axis found perpendicular to C{}\n'.format(Cn, Cn))
                if sigma_h(symbols, coords, hsa, tol=tol):
                    if not quiet:
                        f.write('Sigma h found\n')
                    return "D{}h".format(Cn)
                if not quiet:
                    f.write('No sigma h\n')
                if sigma_v(symbols, coords, hsa, tol=tol):
                    if not quiet:
                        f.write('{} sigma v found\n'.format(Cn))
                    return 'D{}d'.format(Cn)
                if not quiet:
                    f.write('less than {} sigma v found\n'.format(Cn))
                return 'D{}'.format(Cn)

        if not quiet:
            f.write('Less than {} C2 axis found perpendicular to C{}\n'.format(Cn, Cn))

        if sigma_h(symbols, coords, hsa, tol=tol):
            if not quiet:
                f.write('Sigma h found\n')
            return 'C{}h'.format(Cn)
        if not quiet:
            f.write('No sigma h found\n')

        if sigma_v(symbols, coords, hsa, tol=tol):
            if not quiet:
                f.write('{} sigma v found\n'.format(Cn))
            return 'C{}v'.format(Cn)
        if not quiet:
            f.write('Less than {} sigma v found\n'.format(Cn))

        if improper_rotations(Cn, hsa, symbols, coords, tol=tol):
            if not quiet:
                f.write('S2{} found'.format(Cn))
            return 'S2{}'.format(Cn)

    return 'C{}'.format(Cn)



import pyvista as pv
import mendeleev as md


class Atom:

    def __init__(self, symbol, position, resolution, radius_scale=1.5):
        self.symbol = symbol
        self.position = position
        self.resolution = resolution
        self.radius_scale = radius_scale
        self.color = self.get_color()

    @property
    def element(self):
        return md.element(self.symbol)

    @property
    def radius(self):
        return self.element.covalent_radius * 1e-2

    @property
    def plotting_radius(self):
        return self.radius / self.radius_scale

    @property
    def mass(self):
        return self.element.atomic_weight

    def get_color(self):
        return self.element.cpk_color

    @property
    def mesh(self):
        return pv.Sphere(
                radius=self.plotting_radius,
                center=self.position,
                theta_resolution=self.resolution,
                phi_resolution=self.resolution,
                )



import numpy as np
import pyvista as pv
from .atom import Atom
from .bond import Bond
from vasp_suite.graphs import Node, MolGraph
import xyz_py
import multiprocessing as mp

class Molecule:

    def __init__(self, symbols, coords, resolution, scaler, bond_radius, radius_scaler=1.12):
        self.symbols = symbols
        self.coords = coords
        self.resolution = resolution
        self.scaler = scaler
        self.atoms = list(map(Atom, self.symbols, self.coords, [self.resolution] * len(self.symbols), [self.scaler] * len(self.symbols)))
        self.radius_scaler = radius_scaler

        self.bond_radius = bond_radius

    @classmethod
    def from_xyz(cls, filename, resolution, scaler, bond_radius):
        symbols, coords = xyz_py.load_xyz(filename)
        return cls(symbols, coords, resolution, scaler, bond_radius)

    @property
    def graph(self):
        graph = MolGraph(scaler=self.radius_scaler)
        for idx, atom in enumerate(self.atoms):
            node = Node(atom.position, atom.symbol, atom.radius, idx, atom.mass, scaler=self.radius_scaler)
            node.__setattr__('atom', atom)
            graph.add_node(node)
        graph.construct_graph
        return graph

    @property
    def Axes(self):
        coords = np.array(self.coords)
        masses = list(map(lambda x: x.mass, self.atoms))
        inertia_tensor = coords.T @ np.diag(masses) @ coords
        eigvals, eigvecs = np.linalg.eig(inertia_tensor)
        return eigvals, eigvecs

    @property
    def axes_mesh(self):
        origin = [-10, -10, -10]
        axes = self.Axes[1]
        return pv.Arrow(start=origin, direction=axes[:, 0], tip_length=0.1, tip_radius=0.05) + \
                pv.Arrow(start=origin, direction=axes[:, 1], tip_length=0.1, tip_radius=0.05) + \
                pv.Arrow(start=origin, direction=axes[:, 2], tip_length=0.1, tip_radius=0.05)

    @property
    def align(self):
        eigvals, eigvecs = self.Axes
        idx = np.argsort(eigvals)
        self.coords = self.coords @ eigvecs[:, idx]
        for idx, atom in enumerate(self.atoms):
            atom.position = self.coords[idx]

    def rotate(self, R):
        self.coords = self.coords @ R
        for idx, atom in enumerate(self.atoms):
            atom.position = self.coords[idx]

    def filter_bonds(self, bonds):
        """
        Fileter out repeated bonds
        """
        seen = []
        for bond in bonds:
            if bond not in seen:
                seen.append(bond)
                yield bond

    @property
    def bonds(self):
        graph = self.graph
        edges = graph.edges

        bonds = self.create_bonds(edges)
        bonds = list(self.filter_bonds(bonds))
        return bonds

    def process_edge(self, args):
        key, values = args
        bonds = []
        if len(values) == 0:
            return bonds
        elif len(values) == 1:
            atom1 = key.atom
            atom2 = values[0].atom
            bonds.append(Bond(atom1, atom2, self.bond_radius, resolution=self.resolution))
        else:
            for value in values:
                atom1 = key.atom
                atom2 = value.atom
                bonds.append(Bond(atom1, atom2, self.bond_radius, resolution=self.resolution))
        return bonds

    def create_bonds(self, edges):
        with mp.Pool() as pool:
            args = [(key, values) for key, values in edges.items()]
            results = pool.map(self.process_edge, args)
        # Flatten the list of lists
        bonds = [bond for bond_list in results for bond in bond_list]
        return bonds

    @property
    def bounding_box(self):
        coords = np.array(self.coords)
        min_coords = np.min(coords, axis=0) * 1.2
        max_coords = np.max(coords, axis=0) * 1.2
        return min_coords, max_coords



import numpy as np
import mendeleev as md
from vasp_suite.structure import Structure
from vasp_suite.graphs import Node, MolGraph
import multiprocessing as mp
from time import perf_counter

from .atom import Atom
from .bond import Bond


class Crystal():

    def __init__(self, file, resolution, scaler, bond_radius):
        self.resolution = resolution
        self.scaler = scaler
        self.bond_radius = bond_radius
        if file.endswith('.cif'):
            self.structure = Structure.from_cif(file)
        else:
            try:
                self.structure = Structure.from_poscar(file)
            except NotImplementedError:
                raise ValueError('Invalid file format')

    @property
    def atoms(self):
        symbols = self.structure.atom_list
        coords = self.structure.cart_coords
        return list(map(Atom, symbols, coords, [self.resolution]*len(symbols), [self.scaler]*len(symbols)))

    @property
    def graph(self):
        graph = MolGraph()
        for idx, atom in enumerate(self.atoms):
            node = Node(atom.position, atom.symbol, atom.radius, idx, atom.mass, scaler=1.12)
            node.__setattr__('atom', atom)
            graph.add_node(node)
        graph.construct_graph
        return graph

    def filter_bonds(self, bonds):
        """
        Fileter out repeated bonds
        """
        seen = []
        for bond in bonds:
            if bond not in seen:
                seen.append(bond)
                yield bond

    @property
    def bonds(self):
        graph = self.graph
        edges = graph.edges

        bonds = self.create_bonds(edges)
        bonds = list(self.filter_bonds(bonds))
        return bonds

    def process_edge(self, args):
        key, values = args
        bonds = []
        if len(values) == 0:
            return bonds
        elif len(values) == 1:
            atom1 = key.atom
            atom2 = values[0].atom
            bonds.append(Bond(atom1, atom2, self.bond_radius, resolution=self.resolution))
        else:
            for value in values:
                atom1 = key.atom
                atom2 = value.atom
                bonds.append(Bond(atom1, atom2, self.bond_radius, resolution=self.resolution))
        return bonds

    def create_bonds(self, edges):
        with mp.Pool() as pool:
            args = [(key, values) for key, values in edges.items()]
            results = pool.map(self.process_edge, args)
        # Flatten the list of lists
        bonds = [bond for bond_list in results for bond in bond_list]
        return bonds




import numpy as np
from PyQt5 import QtWidgets, QtGui, QtCore

# build the gui window that pops up
# add an panel to the side panel of the main window to host the Ab initio calculations

# the gui section returns effecitively a dictionary of the inputs to run

# create a seperate class to perform, store and report the results of the ab initio calculations

class AbInitioGui(QtWidgets.QDialog):

    def __init__(self, atom_list):
        super().__init__()
        self.setWindowTitle("MolVis Ab Initio")
        self.setGeometry(100, 100, 600, 600)
        self.set_basis_sets = {}
        self.set_ecp = {}
        self.atom_list=atom_list

        for atom in self.atom_list:
            self.set_basis_sets[atom] = 'sto-3g'
            self.set_ecp[atom] = None


        self.initUI()


    def initUI(self):
        layout = QtWidgets.QVBoxLayout()

        self.tabs = QtWidgets.QTabWidget()
        self.tabs_layout = QtWidgets.QGridLayout()
        self.tabs.setLayout(self.tabs_layout)

        self.dft_tab = QtWidgets.QWidget()
        self.scf_tab = QtWidgets.QWidget()
        self.mcscf_tab = QtWidgets.QWidget()

        # DFT tab design
        # basis sets
        # ecp
        # xc functional
        # calculations type (single point, geomopt, freq)
        # RHF, URHF, ...
        # d3bf - vdw
        self.dft_tab_layout = QtWidgets.QGridLayout()

        self.dft_tabs = QtWidgets.QTabWidget()
        self.dft_tabs_layout = QtWidgets.QVBoxLayout()
        self.dft_tabs.setLayout(self.dft_tabs_layout)

        self.basis_set_tab = QtWidgets.QWidget()

        self.basis_set_tab_layout = QtWidgets.QVBoxLayout()

        self.bs_list_widget = QtWidgets.QListWidget()
        for atom in self.atom_list:
            self.bs_list_widget.addItem(atom)
        self.basis_set_tab_layout.addWidget(self.bs_list_widget)

        self.bs_selector_label = QtWidgets.QLabel('Basis Set')
        self.bs_selector = QtWidgets.QComboBox()
        self.bs_selector.addItem('aug-cc-PVDZ')
        self.ecp_selector_label = QtWidgets.QLabel('Effective Core Potential')
        self.ecp_selector = QtWidgets.QComboBox()
        self.ecp_selector.addItem('None')

        self.bs_set_button = QtWidgets.QPushButton('Set')
        self.bs_set_button.clicked.connect(self.set_basis)

        self.basis_set_tab_layout.addWidget(self.bs_selector_label)
        self.basis_set_tab_layout.addWidget(self.bs_selector)
        self.basis_set_tab_layout.addWidget(self.ecp_selector_label)
        self.basis_set_tab_layout.addWidget(self.ecp_selector)
        self.basis_set_tab_layout.addWidget(self.bs_set_button)

        self.basis_set_tab.setLayout(self.basis_set_tab_layout)

        self.calc_params_tab = QtWidgets.QWidget()

        self.calc_params_tab_layout = QtWidgets.QVBoxLayout()

        # functional
        # charge
        # spin
        # symmetry
        self.func_label = QtWidgets.QLabel('xc Functional')
        self.func_selector = QtWidgets.QComboBox()
        self.func_selector.addItem('pbe')

        self.charge_label = QtWidgets.QLabel('Charge')
        self.charge_input = QtWidgets.QLineEdit()
        self.charge_input.setValidator(QtGui.QIntValidator())
        self.charge_input.setPlaceholderText('0')

        self.spin_label = QtWidgets.QLabel('Spin')
        self.spin_input = QtWidgets.QLineEdit()
        self.spin_input.setValidator(QtGui.QDoubleValidator())
        self.spin_input.setPlaceholderText('0.0')

        self.symmetry_check_box = QtWidgets.QCheckBox('Symmetry')

        self.calc_params_tab_layout.addWidget(self.func_label)
        self.calc_params_tab_layout.addWidget(self.func_selector)
        self.calc_params_tab_layout.addWidget(self.charge_label)
        self.calc_params_tab_layout.addWidget(self.charge_input)
        self.calc_params_tab_layout.addWidget(self.spin_label)
        self.calc_params_tab_layout.addWidget(self.spin_input)
        self.calc_params_tab_layout.addWidget(self.symmetry_check_box)

        self.calc_params_tab.setLayout(self.calc_params_tab_layout)

        self.dft_tabs.addTab(self.calc_params_tab, 'Calculation Parameters')
        self.dft_tabs.addTab(self.basis_set_tab, 'Basis Sets')

        self.dft_tab_layout.addWidget(self.dft_tabs)
        self.dft_tab.setLayout(self.dft_tab_layout)

        # scf tab design
        # self.scf_tab_layout = QtWidgets.QGridLayout()

        self.tabs.addTab(self.dft_tab, 'DFT')
        self.tabs.addTab(self.scf_tab, 'HF')
        self.tabs.addTab(self.mcscf_tab, 'MCSCF')

        layout.addWidget(self.tabs)

        self.button_box = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)
        layout.addWidget(self.button_box)

        self.setLayout(layout)


    def parse_calculation(self):
        calculation_instructions = {}
        current_tab = self.tabs.currentIndex() # 0 is Dft, 1 is SCF, 2 is MCSCF
        if current_tab == 0:

            calculation_instructions['calc'] = 'DFT'
            calculation_instructions['basis'] = self.set_basis_sets
            calculation_instructions['ecp'] = self.set_ecp
            xc = self.func_selector.currentText()
            try:
                charge = float(self.charge_input.text())
            except:
                charge = 0
            try:
                spin = float(self.spin_input.text())
            except:
                spin = 0.0
            symm = self.symmetry_check_box.isChecked()

            calculation_instructions['xc'] = xc
            calculation_instructions['charge'] = charge
            calculation_instructions['spin'] = spin
            calculation_instructions['symmetry'] = symm

            return calculation_instructions


        else:
            return None

    def set_basis(self):
        if self.bs_list_widget.currentItem() is not None:
            atom = self.bs_list_widget.currentItem().text()
            self.set_basis_sets[atom] = self.bs_selector.currentText()
            self.set_ecp[atom] = self.ecp_selector.currentText()
            print(self.set_basis_sets)
            print(self.set_ecp)
        else:
            return



import pyvista as pv
import numpy as np
from .atom import Atom
from time import perf_counter
import functools


class Bond:

    def __init__(self, atom1, atom2, radius, resolution=200):
        self.atom1 = atom1
        self.atom2 = atom2
        self.resolution = int(resolution / 2)
        self.radius = radius
        self.center
        self.centers
        self.direction
        self.height
        self.heights
        self.ratios

    @property
    def color(self):
        return 'grey'

    @functools.cached_property
    def center(self):
        return (self.atom1.position + self.atom2.position) / 2

    @functools.cached_property
    def direction(self):
        return self.atom2.position - self.atom1.position

    @functools.cached_property
    def height(self):
        return np.linalg.norm(self.direction)

    @functools.cached_property
    def ratios(self):
        ratio12 = self.atom1.radius / (self.atom1.radius + self.atom2.radius)
        ratio21 = self.atom2.radius / (self.atom1.radius + self.atom2.radius)
        return ratio12, ratio21

    @functools.cached_property
    def centers(self):
        # find the point that is (ratio12 / 2) away from atom1
        ratio12, ratio21 = self.ratios
        center1 = self.atom1.position + (ratio12 / 2) * self.direction
        center2 = self.atom2.position - (ratio21 / 2) * self.direction
        return center1, center2

    @functools.cached_property
    def heights(self):
        ratio12, ratio21 = self.ratios
        height1 = self.height * ratio12
        height2 = self.height * ratio21
        return height1, height2

    @property
    def colored_mesh(self):
        color1 = self.atom1.color
        color2 = self.atom2.color

        center1, center2 = self.centers
        height1, height2 = self.heights

        direction = self.direction

        mesh1 = pv.Capsule(
                center=center1,
                direction=direction,
                radius=self.radius,
                cylinder_length=height1,
                resolution=self.resolution,
                )
        mesh2 = pv.Capsule(
                center=center2,
                direction=direction,
                radius=self.radius,
                cylinder_length=height2,
                resolution=self.resolution,
                )
        return (mesh1, color1), (mesh2, color2)

    @functools.cached_property
    def mesh(self):
        """
        returns one cylinder mesh for the bond
        """
        bond =  pv.Capsule(
            center=self.center,
            direction=self.direction,
            radius=self.radius,
            cylinder_length=self.height,
            resolution=self.resolution
        )
        return bond

    def __eq__(self, other):
        return np.allclose(self.center, other.center)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import setuptools

__version__ = '0.3.0'

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
        name='PyMolVis',
        version=__version__,
        author='W. T. Morrillo',
        description='A program to visualize, create, and generate high-quality images of molecules.',
        long_description=long_description,
        long_description_content_type='text/markdown',
        url="https://gitlab.com/williammorrillo/molvis",
        project_urls={
            "Bug Tracker": "https://gitlab.com/williammorrillo/molvis/-/issues",
            "Documentation": "https://gitlab.com/williammorrillo.gitlab.io/molvis"
        },
        classifiers=[
            'Programming Language :: Python :: 3',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: OS Independent',
        ],
        packages=setuptools.find_packages(),
        python_requires='>=3.6',
        install_requires=[
            'numpy',
            'PyQt5',
            'pyvista',
            'pyvistaqt',
            'xyz_py',
            'vasp_suite',
            'Cython',
            'spin_phonon_suite',
            'gaussian_suite',
            'mendeleev',
            'ase',
            'scipy',
            'seaborn',
            'pyscf'
        ],
        entry_points={
            'console_scripts': [
                'PyMolVis = pymolvis.PyMolVis:main',
            ],
        },
)



import numpy as np
import pyvista as pv
from .atom import Atom


class Surface:

    def __init__(self, symbol, center, width, length, z_level, packing, resolution):
        self.symbol = symbol
        self.center = center
        self.width = width
        self.z_level = z_level
        self.length = length
        self.packing = packing
        self.resolution = resolution

        self.unit_position = np.array([-self.width/2, -self.length/2, self.z_level])
        self.unit = None

    available_packings = ['fcc', 'bcc']

    def check_packing(self):
        if self.packing not in self.available_packings:
            raise ValueError(f"Invalid packing: {self.packing}")

    @property
    def generate_unit(self):
        self.make_unit()

    def make_unit(self):
        # initial atom
        atom = Atom(self.symbol, self.unit_position, resolution=self.resolution)
        atom_2 = Atom(self.symbol, self.unit_position + np.array([(atom.radius * np.sqrt(2))/2, (atom.radius * np.sqrt(2))/2, (-atom.radius * np.sqrt(2))/2]), resolution=self.resolution)
        self.color = atom.color
        self.unit = atom.mesh.merge(atom_2.mesh)
        self.unit_width = atom.radius * np.sqrt(2)
        self.unit_length = atom.radius * np.sqrt(2)

    def create_surface(self):
        if self.unit is None:
            self.generate_unit

        units_in_width = int(self.width / self.unit_width)
        units_in_length = int(self.length / self.unit_length)

        print("adding {} units".format(units_in_width * units_in_length))

        # tile the width
        self.surface = self.unit
        for i in range(units_in_width):
            for j in range(units_in_length):
                # for atom in self.unit:
                    # position = atom.position + np.array([i * self.unit_width, j * self.unit_length, 0])
                    # self.surface.append(Atom(self.symbol, position, resolution=self.resolution))
                self.surface = self.surface.merge(self.unit.copy().translate([i * self.unit_width, j * self.unit_length, 0]))

        print(type(self.surface))

        print("surface created")
        self.surface



import numpy as np
from spin_phonon_suite.vibrations import construct_atomic_basis
from .molecule import Molecule


def electric_distort(atoms, hess, grad_mu, field_vector, resolution, opacity, scaler, bond_radius):
    coords = list(map(lambda x: x.position, atoms))
    symbols = list(map(lambda x: x.symbol, atoms))

    U = construct_atomic_basis(int(grad_mu.shape[0]/3), gaussian_fchk=True,
                               coords=coords)  # units in angstrom
    hess = U.T @ hess @ U

    E = np.array(field_vector)
    grad_mu_E = grad_mu @ E
    r = np.linalg.solve(-1 * hess, U.T @ grad_mu_E)
    r = (U @ r) * 1e10

    coords = coords + r.reshape(-1, 3)

    molecule = Molecule(symbols, coords, resolution, scaler, bond_radius)
    molecule.__setattr__('opacity', opacity)

    return molecule
